@@ -22,10 +22,11 @@
 #include <ElSLib.hxx>
 #include <Geom2dAdaptor_Curve.hxx>
 #include <Geom2dInt_GInter.hxx>
+#include <Geom2d_BSplineCurve.hxx>
 #include <Geom2d_Line.hxx>
 #include <Geom2d_TrimmedCurve.hxx>
 #include <GeomAPI_ProjectPointOnSurf.hxx>
-#include <GeomAdaptor_HSurface.hxx>
+#include <GeomAdaptor_Surface.hxx>
 #include <GeomInt_IntSS.hxx>
 #include <GeomInt_WLApprox.hxx>
 #include <GeomLib_Check2dBSplineCurve.hxx>
@@ -54,10 +55,11 @@
 #include <TopoDS.hxx>
 #include <TopoDS_Edge.hxx>
 #include <gp_Elips.hxx>
+#include <ApproxInt_KnotTools.hxx>
 
 static 
-  void Parameters(const Handle(GeomAdaptor_HSurface)&,
-                  const Handle(GeomAdaptor_HSurface)&,
+  void Parameters(const Handle(GeomAdaptor_Surface)&,
+                  const Handle(GeomAdaptor_Surface)&,
                   const gp_Pnt&,
                   Standard_Real&,
                   Standard_Real&,
@@ -90,8 +92,8 @@ static
   Standard_Boolean  ApproxWithPCurves(const gp_Cylinder& theCyl, 
                                       const gp_Sphere& theSph);
 
-static void  PerformPlanes(const Handle(GeomAdaptor_HSurface)& theS1,
-                           const Handle(GeomAdaptor_HSurface)& theS2,
+static void  PerformPlanes(const Handle(GeomAdaptor_Surface)& theS1,
+                           const Handle(GeomAdaptor_Surface)& theS2,
                            const Standard_Real TolF1,
                            const Standard_Real TolF2,
                            const Standard_Real TolAng,
@@ -101,22 +103,22 @@ static void  PerformPlanes(const Handle(GeomAdaptor_HSurface)& theS1,
                            IntTools_SequenceOfCurves& theSeqOfCurve,
                            Standard_Boolean& theTangentFaces);
 
-static Standard_Boolean ClassifyLin2d(const Handle(GeomAdaptor_HSurface)& theS, 
+static Standard_Boolean ClassifyLin2d(const Handle(GeomAdaptor_Surface)& theS, 
                                       const gp_Lin2d& theLin2d, 
                                       const Standard_Real theTol,
                                       Standard_Real& theP1, 
                                       Standard_Real& theP2);
 //
 static
-  void ApproxParameters(const Handle(GeomAdaptor_HSurface)& aHS1,
-                        const Handle(GeomAdaptor_HSurface)& aHS2,
+  void ApproxParameters(const Handle(GeomAdaptor_Surface)& aHS1,
+                        const Handle(GeomAdaptor_Surface)& aHS2,
                         Standard_Integer& iDegMin,
                         Standard_Integer& iNbIter,
                         Standard_Integer& iDegMax);
 
 static
-  void Tolerances(const Handle(GeomAdaptor_HSurface)& aHS1,
-                  const Handle(GeomAdaptor_HSurface)& aHS2,
+  void Tolerances(const Handle(GeomAdaptor_Surface)& aHS1,
+                  const Handle(GeomAdaptor_Surface)& aHS2,
                   Standard_Real& aTolTang);
 
 static
@@ -165,8 +167,8 @@ IntTools_FaceFace::IntTools_FaceFace()
   myIsDone=Standard_False;
   myTangentFaces=Standard_False;
   //
-  myHS1 = new GeomAdaptor_HSurface ();
-  myHS2 = new GeomAdaptor_HSurface ();
+  myHS1 = new GeomAdaptor_Surface ();
+  myHS2 = new GeomAdaptor_Surface ();
   myTolF1 = 0.;
   myTolF2 = 0.;
   myTol = 0.;
@@ -355,8 +357,9 @@ static Standard_Boolean isTreatAnalityc(const BRepAdaptor_Surface& theBAS1,
 //function : Perform
 //purpose  : intersect surfaces of the faces
 //=======================================================================
-void IntTools_FaceFace::Perform(const TopoDS_Face& aF1,
-                                const TopoDS_Face& aF2)
+void IntTools_FaceFace::Perform (const TopoDS_Face& aF1,
+                                 const TopoDS_Face& aF2,
+                                 const Standard_Boolean theToRunParallel)
 {
   if (myContext.IsNull()) {
     myContext=new IntTools_Context;
@@ -425,12 +428,10 @@ void IntTools_FaceFace::Perform(const TopoDS_Face& aF1,
     Standard_Real umin, umax, vmin, vmax;
     //
     myContext->UVBounds(myFace1, umin, umax, vmin, vmax);
-    CorrectPlaneBoundaries(umin, umax, vmin, vmax);
-    myHS1->ChangeSurface().Load(S1, umin, umax, vmin, vmax);
+    myHS1->Load(S1, umin, umax, vmin, vmax);
     //
     myContext->UVBounds(myFace2, umin, umax, vmin, vmax);
-    CorrectPlaneBoundaries(umin, umax, vmin, vmax);
-    myHS2->ChangeSurface().Load(S2, umin, umax, vmin, vmax);
+    myHS2->Load(S2, umin, umax, vmin, vmax);
     //
     Standard_Real TolAng = 1.e-8;
     //
@@ -464,11 +465,11 @@ void IntTools_FaceFace::Perform(const TopoDS_Face& aF1,
     // F1
     myContext->UVBounds(myFace1, umin, umax, vmin, vmax); 
     CorrectPlaneBoundaries(umin, umax, vmin, vmax);
-    myHS1->ChangeSurface().Load(S1, umin, umax, vmin, vmax);
+    myHS1->Load(S1, umin, umax, vmin, vmax);
     // F2
     myContext->UVBounds(myFace2, umin, umax, vmin, vmax);
     CorrectSurfaceBoundaries(myFace2, myTol * 2., umin, umax, vmin, vmax);
-    myHS2->ChangeSurface().Load(S2, umin, umax, vmin, vmax);
+    myHS2->Load(S2, umin, umax, vmin, vmax);
   }
   else if ((aType2==GeomAbs_Plane) && isFace1Quad)
   {
@@ -476,21 +477,21 @@ void IntTools_FaceFace::Perform(const TopoDS_Face& aF1,
     //F1
     myContext->UVBounds(myFace1, umin, umax, vmin, vmax);
     CorrectSurfaceBoundaries(myFace1, myTol * 2., umin, umax, vmin, vmax);
-    myHS1->ChangeSurface().Load(S1, umin, umax, vmin, vmax);
+    myHS1->Load(S1, umin, umax, vmin, vmax);
     // F2
     myContext->UVBounds(myFace2, umin, umax, vmin, vmax);
     CorrectPlaneBoundaries(umin, umax, vmin, vmax);
-    myHS2->ChangeSurface().Load(S2, umin, umax, vmin, vmax);
+    myHS2->Load(S2, umin, umax, vmin, vmax);
   }
   else
   {
     Standard_Real umin, umax, vmin, vmax;
     myContext->UVBounds(myFace1, umin, umax, vmin, vmax);
     CorrectSurfaceBoundaries(myFace1, myTol * 2., umin, umax, vmin, vmax);
-    myHS1->ChangeSurface().Load(S1, umin, umax, vmin, vmax);
+    myHS1->Load(S1, umin, umax, vmin, vmax);
     myContext->UVBounds(myFace2, umin, umax, vmin, vmax);
     CorrectSurfaceBoundaries(myFace2, myTol * 2., umin, umax, vmin, vmax);
-    myHS2->ChangeSurface().Load(S2, umin, umax, vmin, vmax);
+    myHS2->Load(S2, umin, umax, vmin, vmax);
   }
 
   const Handle(IntTools_TopolTool) dom1 = new IntTools_TopolTool(myHS1);
@@ -502,7 +503,7 @@ void IntTools_FaceFace::Perform(const TopoDS_Face& aF1,
   Tolerances(myHS1, myHS2, TolTang);
 
   {
-    const Standard_Real UVMaxStep = 0.001;
+    const Standard_Real UVMaxStep = IntPatch_Intersection::DefineUVMaxStep(myHS1, dom1, myHS2, dom2);
     const Standard_Real Deflection = 0.1;
     myIntersector.SetTolerances(TolArc, TolTang, UVMaxStep, Deflection); 
   }
@@ -536,7 +537,7 @@ void IntTools_FaceFace::Perform(const TopoDS_Face& aF1,
         Sprintf(aBuff, "%s -p %+10.20f %+10.20f %+10.20f %+10.20f", aBuff, u1, v1, u2, v2);
       }
 
-      cout << aBuff << endl;
+      std::cout << aBuff << std::endl;
     }
 #endif
 
@@ -561,7 +562,7 @@ void IntTools_FaceFace::Perform(const TopoDS_Face& aF1,
       MakeCurve(i, dom1, dom2, TolArc);
     }
     //
-    ComputeTolReached3d();
+    ComputeTolReached3d (theToRunParallel);
     //
     if (bReverse) {
       Handle(Geom2d_Curve) aC2D1, aC2D2;
@@ -625,7 +626,7 @@ void IntTools_FaceFace::Perform(const TopoDS_Face& aF1,
 //function :ComputeTolReached3d 
 //purpose  : 
 //=======================================================================
-void IntTools_FaceFace::ComputeTolReached3d()
+void IntTools_FaceFace::ComputeTolReached3d (const Standard_Boolean theToRunParallel)
 {
   Standard_Integer i, j, aNbLin = mySeqOfCurve.Length();
   if (!aNbLin) {
@@ -635,8 +636,8 @@ void IntTools_FaceFace::ComputeTolReached3d()
   // Minimal tangential tolerance for the curve
   Standard_Real aTolFMax = Max(myTolF1, myTolF2);
   //
-  const Handle(Geom_Surface)& aS1 = myHS1->ChangeSurface().Surface();
-  const Handle(Geom_Surface)& aS2 = myHS2->ChangeSurface().Surface();
+  const Handle(Geom_Surface)& aS1 = myHS1->Surface();
+  const Handle(Geom_Surface)& aS2 = myHS2->Surface();
   //
   for (i = 1; i <= aNbLin; ++i)
   {
@@ -663,8 +664,7 @@ void IntTools_FaceFace::ComputeTolReached3d()
         // Look for the maximal deviation between 3D and 2D curves
         Standard_Real aD, aT;
         const Handle(Geom_Surface)& aS = !j ? aS1 : aS2;
-        if (IntTools_Tools::ComputeTolerance
-            (aC3D, aC2D, aS, aFirst, aLast, aD, aT))
+        if (IntTools_Tools::ComputeTolerance (aC3D, aC2D, aS, aFirst, aLast, aD, aT, Precision::PConfusion(), theToRunParallel))
         {
           if (aD > aTolC)
           {
@@ -714,8 +714,8 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
   IntPatch_IType typl;
   Handle(Geom_Curve) newc;
   //
-  const Standard_Real TOLCHECK   =0.0000001;
-  const Standard_Real TOLANGCHECK=0.1;
+  const Standard_Real TOLCHECK    = 1.e-7;
+  const Standard_Real TOLANGCHECK = 1.e-6;
   //
   rejectSurface = Standard_False;
   reApprox = Standard_False;
@@ -818,18 +818,25 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
           aCurve.SetTolerance(aTolC);
         }
         //
-        aCurve.SetCurve(new Geom_TrimmedCurve(newc, fprm, lprm));
         if(myApprox1) { 
           Handle (Geom2d_Curve) C2d;
           GeomInt_IntSS::BuildPCurves(fprm, lprm, Tolpc,
-                myHS1->ChangeSurface().Surface(), newc, C2d);
+                myHS1->Surface(), newc, C2d);
+
+          if (C2d.IsNull())
+            continue;
+
           aCurve.SetFirstCurve2d(new Geom2d_TrimmedCurve(C2d, fprm, lprm));
         }
         //
         if(myApprox2) { 
           Handle (Geom2d_Curve) C2d;
           GeomInt_IntSS::BuildPCurves(fprm, lprm, Tolpc,
-                    myHS2->ChangeSurface().Surface(), newc, C2d);
+                    myHS2->Surface(), newc, C2d);
+
+          if (C2d.IsNull())
+            continue;
+
           aCurve.SetSecondCurve2d(new Geom2d_TrimmedCurve(C2d, fprm, lprm));
         }
         //
@@ -982,14 +989,14 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
           if(myApprox1) { 
             Handle (Geom2d_Curve) C2d;
             GeomInt_IntSS::BuildPCurves(fprm, lprm, Tolpc, 
-                        myHS1->ChangeSurface().Surface(), newc, C2d);
+                        myHS1->Surface(), newc, C2d);
             aCurve.SetFirstCurve2d(C2d);
           }
 
           if(myApprox2) { 
             Handle (Geom2d_Curve) C2d;
             GeomInt_IntSS::BuildPCurves(fprm,lprm,Tolpc,
-                    myHS2->ChangeSurface().Surface(),newc,C2d);
+                    myHS2->Surface(),newc,C2d);
             aCurve.SetSecondCurve2d(C2d);
           }
         }
@@ -1013,14 +1020,14 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
             if(myApprox1) { 
               Handle (Geom2d_Curve) C2d;
               GeomInt_IntSS::BuildPCurves(fprm,lprm,Tolpc,
-                    myHS1->ChangeSurface().Surface(),newc,C2d);
+                    myHS1->Surface(),newc,C2d);
               aCurve.SetFirstCurve2d(C2d);
             }
 
             if(myApprox2) { 
               Handle (Geom2d_Curve) C2d;
               GeomInt_IntSS::BuildPCurves(fprm,lprm,Tolpc,
-                    myHS2->ChangeSurface().Surface(),newc,C2d);
+                    myHS2->Surface(),newc,C2d);
               aCurve.SetSecondCurve2d(C2d);
             }
             //
@@ -1051,14 +1058,14 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
               if(myApprox1) { 
                 Handle (Geom2d_Curve) C2d;
                 GeomInt_IntSS::BuildPCurves(fprm, lprm, Tolpc, 
-                        myHS1->ChangeSurface().Surface(), newc, C2d);
+                        myHS1->Surface(), newc, C2d);
                 aCurve.SetFirstCurve2d(C2d);
               }
 
               if(myApprox2) { 
                 Handle (Geom2d_Curve) C2d;
                 GeomInt_IntSS::BuildPCurves(fprm, lprm, Tolpc,
-                        myHS2->ChangeSurface().Surface(), newc, C2d);
+                        myHS2->Surface(), newc, C2d);
                 aCurve.SetSecondCurve2d(C2d);
               }
             }//  end of if (typl == IntPatch_Circle || typl == IntPatch_Ellipse)
@@ -1137,17 +1144,17 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
       tol2d = myTolApprox;
 
       GeomAbs_SurfaceType typs1, typs2;
-      typs1 = myHS1->Surface().GetType();
-      typs2 = myHS2->Surface().GetType();
+      typs1 = myHS1->GetType();
+      typs2 = myHS2->GetType();
       Standard_Boolean anWithPC = Standard_True;
 
       if(typs1 == GeomAbs_Cylinder && typs2 == GeomAbs_Sphere) {
         anWithPC = 
-          ApproxWithPCurves(myHS1->Surface().Cylinder(), myHS2->Surface().Sphere());
+          ApproxWithPCurves(myHS1->Cylinder(), myHS2->Sphere());
       }
       else if (typs1 == GeomAbs_Sphere && typs2 == GeomAbs_Cylinder) {
         anWithPC = 
-          ApproxWithPCurves(myHS2->Surface().Cylinder(), myHS1->Surface().Sphere());
+          ApproxWithPCurves(myHS2->Cylinder(), myHS1->Sphere());
       }
       //
       if(!anWithPC) {
@@ -1158,22 +1165,6 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
         tol2d = myTolApprox;
       }
         
-      if(myHS1 == myHS2) { 
-        theapp3d.SetParameters(myTolApprox, tol2d, 4, 8, 0, 30, Standard_False, aParType);
-        rejectSurface = Standard_True;
-      }
-      else { 
-        if(reApprox && !rejectSurface)
-          theapp3d.SetParameters(myTolApprox, tol2d, 4, 8, 0, 30, Standard_False, aParType);
-        else {
-          Standard_Integer iDegMax, iDegMin, iNbIter;
-          //
-          ApproxParameters(myHS1, myHS2, iDegMin, iDegMax, iNbIter);
-          theapp3d.SetParameters(myTolApprox, tol2d, iDegMin, iDegMax,
-                                                  iNbIter, 30, Standard_True, aParType);
-        }
-      }
-      //
       Standard_Real aReachedTol = Precision::Confusion();
       bIsDecomposited = IntTools_WLineTool::
         DecompositionOfWLine(WL,
@@ -1221,6 +1212,35 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
             ilprm = (Standard_Integer)lprm;
           }
         }
+
+        Standard_Boolean anApprox = myApprox;
+        if (typs1 == GeomAbs_Plane) {
+          anApprox = Standard_False;
+          anApprox1 = Standard_True;
+        }
+        else if (typs2 == GeomAbs_Plane) {
+          anApprox = Standard_False;
+          anApprox2 = Standard_True;
+        }
+
+        aParType = ApproxInt_KnotTools::DefineParType(WL, ifprm, ilprm, 
+                                                   anApprox, anApprox1, anApprox2);
+        if (myHS1 == myHS2) {
+          theapp3d.SetParameters(myTolApprox, tol2d, 4, 8, 0, 30, Standard_False, aParType);
+          rejectSurface = Standard_True;
+        }
+        else {
+          if (reApprox && !rejectSurface)
+            theapp3d.SetParameters(myTolApprox, tol2d, 4, 8, 0, 30, Standard_False, aParType);
+          else {
+            Standard_Integer iDegMax, iDegMin, iNbIter;
+            //
+            ApproxParameters(myHS1, myHS2, iDegMin, iDegMax, iNbIter);
+            theapp3d.SetParameters(myTolApprox, tol2d, iDegMin, iDegMax,
+              iNbIter, 30, Standard_True, aParType);
+          }
+        }
+
         //-- lbr : 
         //-- Si une des surfaces est un plan , on approxime en 2d
         //-- sur cette surface et on remonte les points 2d en 3d.
@@ -1292,7 +1312,7 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
               TColgp_Array1OfPnt   tpoles(1,nbpoles);
               
               mbspc.Curve(1,tpoles2d);
-              const gp_Pln&  Pln = myHS1->Surface().Plane();
+              const gp_Pln&  Pln = myHS1->Plane();
               //
               Standard_Integer ik; 
               for(ik = 1; ik<= nbpoles; ik++) { 
@@ -1371,7 +1391,7 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
               TColgp_Array1OfPnt2d tpoles2d(1,nbpoles);
               TColgp_Array1OfPnt   tpoles(1,nbpoles);
               mbspc.Curve((myApprox1==Standard_True)? 2 : 1,tpoles2d);
-              const gp_Pln&  Pln = myHS2->Surface().Plane();
+              const gp_Pln&  Pln = myHS2->Plane();
               //
               Standard_Integer ik; 
               for(ik = 1; ik<= nbpoles; ik++) { 
@@ -1517,7 +1537,7 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
                   Handle(Geom2d_Curve) C2d;
                   Standard_Real aTol = myTolApprox;
                   GeomInt_IntSS::BuildPCurves(fprm, lprm, aTol,
-                            myHS1->ChangeSurface().Surface(), BS, C2d);
+                            myHS1->Surface(), BS, C2d);
                   BS1 = Handle(Geom2d_BSplineCurve)::DownCast(C2d);
                   aCurve.SetFirstCurve2d(BS1);
                 }
@@ -1548,7 +1568,7 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
                   Handle(Geom2d_Curve) C2d;
                   Standard_Real aTol = myTolApprox;
                   GeomInt_IntSS::BuildPCurves(fprm, lprm, aTol,
-                            myHS2->ChangeSurface().Surface(), BS, C2d);
+                            myHS2->Surface(), BS, C2d);
                   BS2 = Handle(Geom2d_BSplineCurve)::DownCast(C2d);
                   aCurve.SetSecondCurve2d(BS2);
                 }
@@ -1677,8 +1697,8 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
 //function : Parameters
 //purpose  : 
 //=======================================================================
- void Parameters(const Handle(GeomAdaptor_HSurface)& HS1,
-                 const Handle(GeomAdaptor_HSurface)& HS2,
+ void Parameters(const Handle(GeomAdaptor_Surface)& HS1,
+                 const Handle(GeomAdaptor_Surface)& HS2,
                  const gp_Pnt& Ptref,
                  Standard_Real& U1,
                  Standard_Real& V1,
@@ -1687,44 +1707,44 @@ void IntTools_FaceFace::MakeCurve(const Standard_Integer Index,
 {
 
   IntSurf_Quadric quad1,quad2;
-  GeomAbs_SurfaceType typs = HS1->Surface().GetType();
+  GeomAbs_SurfaceType typs = HS1->GetType();
 
   switch (typs) {
   case GeomAbs_Plane:
-    quad1.SetValue(HS1->Surface().Plane());
+    quad1.SetValue(HS1->Plane());
     break;
   case GeomAbs_Cylinder:
-    quad1.SetValue(HS1->Surface().Cylinder());
+    quad1.SetValue(HS1->Cylinder());
     break;
   case GeomAbs_Cone:
-    quad1.SetValue(HS1->Surface().Cone());
+    quad1.SetValue(HS1->Cone());
     break;
   case GeomAbs_Sphere:
-    quad1.SetValue(HS1->Surface().Sphere());
+    quad1.SetValue(HS1->Sphere());
     break;
   case GeomAbs_Torus:
-    quad1.SetValue(HS1->Surface().Torus());
+    quad1.SetValue(HS1->Torus());
     break;
   default:
     throw Standard_ConstructionError("GeomInt_IntSS::MakeCurve");
   }
   
-  typs = HS2->Surface().GetType();
+  typs = HS2->GetType();
   switch (typs) {
   case GeomAbs_Plane:
-    quad2.SetValue(HS2->Surface().Plane());
+    quad2.SetValue(HS2->Plane());
     break;
   case GeomAbs_Cylinder:
-    quad2.SetValue(HS2->Surface().Cylinder());
+    quad2.SetValue(HS2->Cylinder());
     break;
   case GeomAbs_Cone:
-    quad2.SetValue(HS2->Surface().Cone());
+    quad2.SetValue(HS2->Cone());
     break;
   case GeomAbs_Sphere:
-    quad2.SetValue(HS2->Surface().Sphere());
+    quad2.SetValue(HS2->Sphere());
     break;
   case GeomAbs_Torus:
-    quad2.SetValue(HS2->Surface().Torus());
+    quad2.SetValue(HS2->Torus());
     break;
   default:
     throw Standard_ConstructionError("GeomInt_IntSS::MakeCurve");
@@ -1886,6 +1906,7 @@ Handle(Geom_Curve) MakeBSpline  (const Handle(IntPatch_WLine)& WL,
     enlarge=Standard_True;
   }
   //
+
   if(!isuperiodic && enlarge) {
 
     if(!Precision::IsInfinite(theumin) &&
@@ -1901,6 +1922,7 @@ Handle(Geom_Curve) MakeBSpline  (const Handle(IntPatch_WLine)& WL,
     else
       theumax = usup;
   }
+
   //
   if(!isvperiodic && enlarge) {
     if(!Precision::IsInfinite(thevmin) &&
@@ -1918,6 +1940,7 @@ Handle(Geom_Curve) MakeBSpline  (const Handle(IntPatch_WLine)& WL,
       thevmax = vsup;
     }
   }
+
   //
   if(isuperiodic || isvperiodic) {
     Standard_Boolean correct = Standard_False;
@@ -2173,8 +2196,8 @@ Standard_Boolean ApproxWithPCurves(const gp_Cylinder& theCyl,
 //function : PerformPlanes
 //purpose  : 
 //=======================================================================
-void  PerformPlanes(const Handle(GeomAdaptor_HSurface)& theS1,
-                    const Handle(GeomAdaptor_HSurface)& theS2,
+void  PerformPlanes(const Handle(GeomAdaptor_Surface)& theS1,
+                    const Handle(GeomAdaptor_Surface)& theS2,
                     const Standard_Real TolF1,
                     const Standard_Real TolF2,
                     const Standard_Real TolAng,
@@ -2185,8 +2208,8 @@ void  PerformPlanes(const Handle(GeomAdaptor_HSurface)& theS1,
                     Standard_Boolean& theTangentFaces)
 {
 
-  gp_Pln aPln1 = theS1->Surface().Plane();
-  gp_Pln aPln2 = theS2->Surface().Plane();
+  gp_Pln aPln1 = theS1->Plane();
+  gp_Pln aPln2 = theS2->Plane();
 
   IntAna_QuadQuadGeo aPlnInter(aPln1, aPln2, TolAng, TolTang);
 
@@ -2304,7 +2327,7 @@ static inline Standard_Boolean COINC(const Standard_Real d1,
 {
   return (d1 <= tol && d1 >= -tol) && (d2 <= tol && d2 >= -tol);
 }
-Standard_Boolean ClassifyLin2d(const Handle(GeomAdaptor_HSurface)& theS, 
+Standard_Boolean ClassifyLin2d(const Handle(GeomAdaptor_Surface)& theS, 
                                const gp_Lin2d& theLin2d, 
                                const Standard_Real theTol,
                                Standard_Real& theP1, 
@@ -2315,10 +2338,10 @@ Standard_Boolean ClassifyLin2d(const Handle(GeomAdaptor_HSurface)& theS,
   Standard_Real par[2];
   Standard_Integer nbi = 0;
 
-  xmin = theS->Surface().FirstUParameter();
-  xmax = theS->Surface().LastUParameter();
-  ymin = theS->Surface().FirstVParameter();
-  ymax = theS->Surface().LastVParameter();
+  xmin = theS->FirstUParameter();
+  xmax = theS->LastUParameter();
+  ymin = theS->FirstVParameter();
+  ymax = theS->LastVParameter();
 
   theLin2d.Coefficients(A, B, C);
 
@@ -2439,8 +2462,8 @@ Standard_Boolean ClassifyLin2d(const Handle(GeomAdaptor_HSurface)& theS,
 //function : ApproxParameters
 //purpose  : 
 //=======================================================================
-void ApproxParameters(const Handle(GeomAdaptor_HSurface)& aHS1,
-                      const Handle(GeomAdaptor_HSurface)& aHS2,
+void ApproxParameters(const Handle(GeomAdaptor_Surface)& aHS1,
+                      const Handle(GeomAdaptor_Surface)& aHS2,
                       Standard_Integer& iDegMin,
                       Standard_Integer& iDegMax,
                       Standard_Integer& iNbIter)
@@ -2453,8 +2476,8 @@ void ApproxParameters(const Handle(GeomAdaptor_HSurface)& aHS1,
   iDegMin=4;
   iDegMax=8;
   //
-  aTS1=aHS1->Surface().GetType();
-  aTS2=aHS2->Surface().GetType();
+  aTS1=aHS1->GetType();
+  aTS2=aHS2->GetType();
   //
   // Cylinder/Torus
   if ((aTS1==GeomAbs_Cylinder && aTS2==GeomAbs_Torus) ||
@@ -2465,8 +2488,8 @@ void ApproxParameters(const Handle(GeomAdaptor_HSurface)& aHS1,
     //
     aPC=Precision::Confusion();
     //
-    aCylinder=(aTS1==GeomAbs_Cylinder)? aHS1->Surface().Cylinder() : aHS2->Surface().Cylinder();
-    aTorus=(aTS1==GeomAbs_Torus)? aHS1->Surface().Torus() : aHS2->Surface().Torus();
+    aCylinder=(aTS1==GeomAbs_Cylinder)? aHS1->Cylinder() : aHS2->Cylinder();
+    aTorus=(aTS1==GeomAbs_Torus)? aHS1->Torus() : aHS2->Torus();
     //
     aRC=aCylinder.Radius();
     aRT=aTorus.MinorRadius();
@@ -2487,14 +2510,14 @@ void ApproxParameters(const Handle(GeomAdaptor_HSurface)& aHS1,
 //function : Tolerances
 //purpose  : 
 //=======================================================================
-void Tolerances(const Handle(GeomAdaptor_HSurface)& aHS1,
-                const Handle(GeomAdaptor_HSurface)& aHS2,
+void Tolerances(const Handle(GeomAdaptor_Surface)& aHS1,
+                const Handle(GeomAdaptor_Surface)& aHS2,
                 Standard_Real& aTolTang)
 {
   GeomAbs_SurfaceType aTS1, aTS2;
   //
-  aTS1=aHS1->Surface().GetType();
-  aTS2=aHS2->Surface().GetType();
+  aTS1=aHS1->GetType();
+  aTS2=aHS2->GetType();
   //
   // Cylinder/Torus
   if ((aTS1==GeomAbs_Cylinder && aTS2==GeomAbs_Torus) ||
@@ -2505,8 +2528,8 @@ void Tolerances(const Handle(GeomAdaptor_HSurface)& aHS1,
     //
     aPC=Precision::Confusion();
     //
-    aCylinder=(aTS1==GeomAbs_Cylinder)? aHS1->Surface().Cylinder() : aHS2->Surface().Cylinder();
-    aTorus=(aTS1==GeomAbs_Torus)? aHS1->Surface().Torus() : aHS2->Surface().Torus();
+    aCylinder=(aTS1==GeomAbs_Cylinder)? aHS1->Cylinder() : aHS2->Cylinder();
+    aTorus=(aTS1==GeomAbs_Torus)? aHS1->Torus() : aHS2->Torus();
     //
     aRC=aCylinder.Radius();
     aRT=aTorus.MinorRadius();
